%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  This script handles the condition that
%  parameter mu(t) follows a sine curve at time t 
%  with different phase offset in each category
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all;

% fixing random number seeds
% (you may comment out this region)
rand('seed',150);
randn('seed',100);

% training data properties
% 'N' must be a multiple of 'K'
N = 90;%300
T = 100;
K = 3;%5

% test data properties
% '1<=k<=K' and '1<=t<=T' required
t = 30;
k = 2;

% model parameters
alpha0 = .1;
beta1 = 1;
beta2 = 1;
max_K = 9;%50
iter_count = 100;
min_supp_thr = N/100;

% setting parameters: sine curves
patterns = zeros(K,T);
for i=1:K
    patterns(i,:) = .5 +.4.*sin([1:T].*2.*pi./T+2*pi/K.*i);
end
figure('Name','Ground Truth');
for i=1:K
    subplot(K,1,i);
    plot(patterns(i,:));
    title(sprintf('pattern %d',i));
    axis([1 T 0 1]);
end

% random data generation from the parameters
tr_data = rand(N,T);
tr_data = double(tr_data <...
    (reshape(repmat(patterns,1,N/K)',size(tr_data,2),size(tr_data,1)))');
te_data = rand(1,T);
te_data = double(te_data < patterns(k,:));
te_data_observed = te_data(1:t);
te_data_future = te_data(t+1:end);
figure('Name','Training Data (White: 1, Black: 0)');
image(tr_data.*255);
colormap('Gray')
xlabel('Time of a Day');
ylabel('Day ID');

% estimation
%tic;[Alpha Beta Z md] = npb4go_learn...
tic;[Alpha Beta Z] = npb4go_learn...
    (tr_data,max_K,alpha0,beta1,beta2,iter_count);toc;
sumZ = sum(Z,1);
figure('Name','Clustering Result');
image(Z.*255);
colormap('Gray')
xlabel('Cluster ID');
ylabel('Day ID');

% output modeling results
figure('Name','Category Distribution');
bar(sumZ);
xlabel('Cluster ID');
ylabel('# of days included');
axis([1 max_K 0 max(sumZ)+1]);
idx = find(sumZ > min_supp_thr);
idx = sortrows([sumZ(idx)' idx'],-1)';
idx = idx(2,:);
figure('Name','Estimated Patterns')
for i=1:length(idx)
    subplot(length(idx),1,i);
    plot(Beta(:,idx(i),1)./sum(Beta(:,idx(i),:),3));
    title(sprintf('pattern k=%d (# of incl. data: %.1f)'...
        ,idx(i),sumZ(idx(i))));
    axis([1 T 0 1]);
end

% prediction
exp_pi = calc_exp_pi(Alpha);
predicted_prob = npb4go_predict(te_data_observed,Beta,exp_pi);

% output prediction results
figure('Name','Prediction Result of New Data');
subplot(4,1,1);
plot(patterns(k,:));
axis([1 T 0 1]);
title('Hidden Parameter');
subplot(4,1,2);
bar(1:t,te_data_observed);
axis([1 T 0 1]);
title(sprintf('Observed Data (1 <= t <= %d)',t));
subplot(4,1,3);
bar(t+1:T,te_data_future);
axis([1 T 0 1]);
title(sprintf('Original Future Observation (%d <= t <= %d)',t+1,T));
subplot(4,1,4);
plot(t+1:T,predicted_prob);
axis([1 T 0 1]);
title(sprintf('Predicted Future Probabilities (%d <= t <= %d)',t+1,T));
