function [Alpha Beta Z E_Z] = npb4go_learn(data,K,alpha0,beta1,beta2,iter_count)

if nargin<6
    iter_count = 100;
end
if nargin<4
    beta1 = 1;
    beta2 = 1;
end
if nargin<3
    alpha0 = .1;
end

N = size(data,1);
T = size(data,2);
Z = zeros(N,K);
Alpha = zeros(2,K);
Beta = zeros(T,K,2);

%% initialization: randomly annotating latent variables
perm = randperm(N);
q = fix(N/K);
for k=1:K
    Z(perm(q*(k-1)+1:q*k),k) = 1;
end
rest = N - q*K;
for i=1:rest
    Z(perm(q*K+i),i) = 1;
end

%% MCMC iteration
for iter=1:iter_count
    % evaluating posterior parameters
    sumZ = sum(Z,1);
    Alpha(1,:) = 1 + sumZ;
    Alpha(2,:) = alpha0 + sum(sumZ) - cumsum(sumZ);    
    Beta(:,:,1) = beta1 + data'*Z;
    Beta(:,:,2) = beta2 + (1-data)'*Z;
    % sampling Mu and Pi
    Mu = betarnd(Beta(:,:,1),Beta(:,:,2));
    V = betarnd(Alpha(1,:),Alpha(2,:));
    V(end) = 1;
    Pi = V.*[1 cumprod(1-V(1:end-1))];
    % sampling Z
    lnPxm = data*log(Mu)+(1-data)*log(1-Mu);
    lnPxm = lnPxm - repmat(max(lnPxm,[],2),1,K);    
    pZ = repmat(Pi,N,1).*exp(lnPxm);
    pZ = pZ ./ repmat(sum(pZ,2),1,K);
    Z = mltrnd(pZ,1);
end

sumZ = sum(Z,1);
Alpha(1,:) = 1 + sumZ;
Alpha(2,:) = alpha0 + sum(sumZ) - cumsum(sumZ);
Beta(:,:,1) = beta1 + data'*Z;
Beta(:,:,2) = beta2 + (1-data)'*Z;

%% calculating E[Z]
if nargout>=4
    P = zeros(T,K,N);
    for n=1:N
        P(:,:,n) = beta(...
            repmat(data(n,:)',1,K)+Beta(:,:,1),1-repmat(data(n,:)',1,K)+Beta(:,:,2))./beta(Beta(:,:,1),Beta(:,:,2));
    end
    E_V = Alpha(1,:)./sum(Alpha,1);
    E_V(end) = 1;
    E_Pi = zeros(1,K,N);
    E_Pi(:,:,:) = repmat((E_V.*[1 cumprod(1-E_V(1:end-1))])',1,N);
    buffer = zeros(size(pZ))';
    buffer(:,:) = prod(P,1).*E_Pi;
    E_Z = buffer';
    E_Z = E_Z ./ repmat(sum(E_Z,2),1,K);
    sumZ = sum(E_Z,1);
end

end